//==========================================================================
//
//   algorithm.h 
//
//==========================================================================
// $Id: algorithm.h,v 1.8 1999/01/19 12:29:47 raitner Exp $

#ifndef GTL_ALGORITHM_H
#define GTL_ALGORITHM_H

#include <GTL/GTL.h>
#include <GTL/graph.h>
#include <string>

__GTL_BEGIN_NAMESPACE

/**
 * @short Abstract baseclass for all algoritm-classes.
 */
class algorithm {
public:

    /**
     * error values returned by algorithms in run and check.
     */
    enum {
	GTL_OK = 1,
	GTL_ERROR = 0
    };

    algorithm () { };
    virtual ~algorithm () { };    

    /**
     * Applies algorithm to graph <code>g</code>.
     * 
     * @param <code>G</code> graph
     * @return <code>algorithm::GTL_OK</code> on success 
     * <code>algorithm::GTL_ERROR</code>
     * otherwise 
     */
    virtual int run (graph& g) = 0;
    
    /**
     * Checks whether all preconditions are satisfied.
     * 
     * @param <code>G</code> graph
     * @return <code>algorithm::GTL_OK</code> on success 
     * <code>algorithm::GTL_ERROR</code>
     * otherwise 
     */
    virtual int check (graph& g) = 0;
    
    /**
     * Resets algorithm, i.e. prepares the algorithm to be applied to
     * another graph. 
     */
    virtual void reset () = 0;
};


__GTL_END_NAMESPACE

#endif // GTL_ALGORITHM_H

//--------------------------------------------------------------------------
//   end of file
//--------------------------------------------------------------------------
