//==========================================================================
//
//   edge.h
//
//==========================================================================
// $Id: edge.h,v 1.9 1999/03/01 10:20:13 raitner Exp $

#ifndef GTL_EDGE_H
#define GTL_EDGE_H

#include <GTL/GTL.h>
#include <list>

__GTL_BEGIN_NAMESPACE
 
//--------------------------------------------------------------------------
//   For MSVC 5.0 edge.h has to be included before node.h and
//   {node,edge}_data.h. So we only declare needed classnames here
//--------------------------------------------------------------------------

class node;
class edge_data;

//--------------------------------------------------------------------------
//   edge
//--------------------------------------------------------------------------

/**
 * @short An edge in a graph
 */
class edge
{
public:
    /**
     * Default constructor. Creates an invalid edge. Example:
     * <pre>
     *   graph g;
     *   node n1, n2;
     *   edge e;
     *
     *   n1 = g.new_node();
     *   n2 = g.new_node();
     *   e = g.new_edge(n1, n2);
     * </pre>
     */
    edge();

    /**
     * Returns the source node of the edge.
     */
    node source() const;

    /**
     * Returns the target node of the edge.
     */
    node target() const;

    /**
     * changes the direction of this edge.
     */

    void reverse ();

    /**
     * @internal
     */
    list<node> sources() const;

    /**
     * @internal
     */
    list<node> targets() const;

    /**
     * @internal
     */
    int id() const;

    
    /**
     * Returns true iff node is hidden.
     */
    bool is_hidden () const;


    //================================================== Implementation
    
private:
    edge_data *data;
    
    void remove_from(int where) const; // 0 = sources, 1 == targets

    friend class graph;
    friend class node;
    
    friend bool operator==(edge, edge);
    friend bool operator!=(edge, edge);
    friend bool operator<(edge, edge);
    friend ostream& operator<< (ostream& os, const edge& e);
};

__GTL_END_NAMESPACE

#endif // GTL_EDGE_H

//--------------------------------------------------------------------------
//   end of file
//--------------------------------------------------------------------------

