//==========================================================================
//
//   gml_scanner.h 
//
//==========================================================================
// $Id: gml_scanner.h,v 1.7 1999/02/08 20:23:17 raitner Exp $

#ifndef GTL_GML_SCANNER_H
#define GTL_GML_SCANNER_H

#include <GTL/GTL.h>
#include <cstdio>

__GTL_BEGIN_NAMESPACE

/*
 * start-size of buffers for reading strings. If too small it will be enlarged
 * dynamically
 */

#define INITIAL_SIZE 1024

typedef enum {
    GML_KEY, GML_INT, GML_DOUBLE, GML_STRING, GML_L_BRACKET, 
    GML_R_BRACKET, GML_END, GML_LIST, GML_ERROR
} GML_value; 


typedef enum {
    GML_UNEXPECTED, GML_SYNTAX, GML_PREMATURE_EOF, GML_TOO_MANY_DIGITS,
    GML_OPEN_BRACKET, GML_TOO_MANY_BRACKETS, GML_OK, GML_FILE_NOT_FOUND
} GML_error_value;


/**
 * @short Reason and position of an error in an GML file
 */
struct GML_error {
    /**
     * Contains the error description as symbolic constant.
     */
    GML_error_value err_num;
    
    /**
     * Contains the line, where the error was detected. This will
     * usually be near the line where the error really is
     * located.
     */
    int line;

    /**
     * Contains the column, where the error was detected.
     */
    int column;
};


union GML_tok_val {
    long integer;
    double floating;
    char* str;
    struct GML_error err;
};

/**
 * @internal
 */
struct GML_token { 
    GML_value kind;
    union GML_tok_val value;
};

/*
 * global variables
 */

extern unsigned int GML_line;
extern unsigned int GML_column;

/*
 * if you are interested in the position where an error occured it is a good
 * idea to set GML_line and GML_column back. 
 * This is what GML_init does.
 */
 
void GML_init ();

/*
 * returns the next token in file. If an error occured it will be stored in 
 * GML_token.
 */

struct GML_token GML_scanner (FILE*);

extern char* GML_table[];

__GTL_END_NAMESPACE

#endif // GTL_GML_SCANNER_H

//--------------------------------------------------------------------------
//   end of file
//--------------------------------------------------------------------------
